#include <windows.h>
#include <stdio.h>
#include "jdesk.h"
#include "../../lssrc/ls-b24/lsapi/lsapi.h"

const char rcsRevision[] = "$Revision: 0.40 $"; // Our Version 
const char rcsId[] = "$Id: jdesk.c,v 0.40 1999/13/09 00:22:00 jugg $"; // The Full RCS ID.
const char szAppName[] = "DesktopBackgroundClass"; // Our window class, etc

HWND hMainWnd; // main window handle
HWND hContWnd; // LS window handle
HWND parent; // Which is actually also LS window handle

char lsPath[256];

jDeskSettings jds;

HINSTANCE dll;
int screenWidth=1024, screenHeight=768;
int waLeft=0, waRight=-50, waTop=0, waBottom=-50;

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
void lButton(WPARAM wParam);
void mButton(WPARAM wParam);
void rButton(WPARAM wParam);
void executeCommand(char *command);
void setWorkArea(void);
void resetWorkArea(void);
void jDeskSetWorkArea(HWND caller, char *args);
void jDeskResetWorkArea(void);


jDeskSettings ReadSettings()
{
	jDeskSettings settings;
	char tmp[MAXLEN];
	char *token;

	GetRCString("jDeskMButton1", settings.MButton1, "!None", MAXLEN);
	GetRCString("jDeskMButton2", settings.MButton2, "!Popup", MAXLEN);
	GetRCString("jDeskMButton3", settings.MButton3, "!None", MAXLEN);
	GetRCString("jDeskCtrlMButton1", settings.CtrlMButton1, "!None", MAXLEN);
	GetRCString("jDeskCtrlMButton2", settings.CtrlMButton2, "!None", MAXLEN);
	GetRCString("jDeskCtrlMButton3", settings.CtrlMButton3, "!None", MAXLEN);
	GetRCString("jDeskShiftMButton1", settings.ShiftMButton1, "!None", MAXLEN);
	GetRCString("jDeskShiftMButton2", settings.ShiftMButton2, "!None", MAXLEN);
	GetRCString("jDeskShiftMButton3", settings.ShiftMButton3, "!None", MAXLEN);

	settings.WorkArea = GetRCBool("jDeskWorkArea", TRUE);
	settings.OnMouseDown = GetRCBool("jDeskOnMouseDown", TRUE);

	GetRCString("jDeskWorkArea", tmp, "0,-50,0,-50", MAXLEN);
	token = strtok(tmp, " ,");
		if (token != NULL)
			waLeft = atoi(token);
	token = strtok(NULL, " ,");
		if (token != NULL)
			waRight = atoi(token);
	token = strtok(NULL, " ,");
		if (token != NULL)
			waTop = atoi(token);
	token = strtok(NULL, " ,");
		if (token != NULL)
			waBottom = atoi(token);

	return settings;
}

int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	WNDCLASS wc;
	UINT Msgs[2];

	dll = dllInst;
	parent = ParentWnd;
	strcpy (lsPath, szPath);

	screenWidth = GetSystemMetrics(SM_CXSCREEN);
	screenHeight = GetSystemMetrics(SM_CYSCREEN);

	jds = ReadSettings();

	// Find the litestep main window
	hContWnd = GetLitestepWnd();

	// Set the work area for maximized applications
	setWorkArea();

	{// Register the jDesk desktop window class
		memset(&wc,0,sizeof(wc));
		wc.hCursor = LoadCursor(NULL, IDC_ARROW);
		wc.lpfnWndProc = WndProc;				// our window procedure
		wc.hInstance = dllInst;					// hInstance of DLL
		wc.lpszClassName = szAppName;		// our window class name
		wc.style = 0;
		if (!RegisterClass(&wc))
		{
			MessageBox(parent,"Error registering window class","jDesk",MB_OK);
			return 1;
		}
	}
	{// Create jDesk desktop window
		hMainWnd = CreateWindowEx(
			WS_EX_TOOLWINDOW,					// exstyles 
			szAppName,								// our window class name
			"",												// use description for a window title
			WS_POPUP|WS_CLIPSIBLINGS|WS_CLIPCHILDREN,
			0, 0,											// position 
			screenWidth,screenHeight,	// width & height of window
			hContWnd,									// parent window 
			NULL,											// no menu
			dll,											// hInstance of DLL
			NULL);										// no window creation data
		if (!hMainWnd)
		{
			MessageBox(parent,"Error creating window","jDesk",MB_OK);
			return 1;
		}
	}

	AddBangCommand("!JDESKSETWORKAREA",jDeskSetWorkArea);
	AddBangCommand("!JDESKRESETWORKAREA",jDeskResetWorkArea);

	{//register message for version info
		Msgs[0] = LM_GETREVID;
		Msgs[1] = 0;
		SendMessage(parent, LM_REGISTERMESSAGE, (WPARAM) hMainWnd, (LPARAM) Msgs);
	}

	//Final desktop window settings and display
	SetWindowLong(hMainWnd, GWL_USERDATA, magicDWord);
	SetWindowPos(hMainWnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE);
	ShowWindow(hMainWnd,SW_SHOWNORMAL);
	SetActiveWindow(hMainWnd);
	return 0;
}

void quitModule(HINSTANCE dllInst)
{ 
	UINT Msgs[2];
    
	resetWorkArea();

	Msgs[0] = LM_GETREVID;
	Msgs[1] = 0;
	SendMessage(parent, LM_UNREGISTERMESSAGE, (WPARAM) hMainWnd, (LPARAM) Msgs);

	RemoveBangCommand("!JDESKSETWORKAREA");
	RemoveBangCommand("!JDESKRESETWORKAREA");

	DestroyWindow(hMainWnd); // delete jDesk desktop window
	UnregisterClass(szAppName,dllInst); // unregister jDesk desktop window class
	return;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case LM_GETREVID:
			{
				char *buf = (char *) lParam;

				if (wParam == 0)
				{
					strcpy(buf, "jdesk.dll: ");
					strcat(buf, &rcsRevision[11]);
					buf[strlen(buf)-1] = '\0';
				}
				else if (wParam == 1)
				{
					strcpy(buf, &rcsId[1]);
					buf[strlen(buf)-1] = '\0';
				}
				else
					strcpy(buf, "");
				return strlen(buf);
			}
		case WM_DISPLAYCHANGE:
			{// Screen resolution has changed. Update stored settings.
				screenWidth = LOWORD(lParam); 
				screenHeight = HIWORD(lParam);
				return 0;
			}
		case WM_ENDSESSION:
		case WM_QUERYENDSESSION:
			return SendMessage(parent,message,wParam,lParam);
		case WM_ERASEBKGND:
			{
				PaintDesktop((HDC)wParam);
				return 1;
			}
		case WM_SYSCOMMAND:
			{
				switch (wParam)
				{
					case SC_CLOSE://For using the standard alt+F4 to shutdown windows
						ParseBangCommand(hMainWnd, "!ShutDown", NULL);
						return 0;
					default:
						break;
				}
				break;
			}
		case WM_KEYDOWN:
		case WM_KEYUP: // forward
				PostMessage(parent,message,wParam,lParam);
			return 0;
		case WM_HOTKEY:
		case WM_WINDOWPOSCHANGING:
			{
				// Trap windowposchanging messages
				WINDOWPOS *c = (WINDOWPOS*)lParam;
				c->hwndInsertAfter = HWND_BOTTOM;
				// c->flags |= SWP_NOZORDER;
				c->flags |= SWP_NOMOVE|SWP_NOSIZE;
				return 0;
			}
		case WM_LBUTTONUP:
			{
				if (!jds.OnMouseDown)
					lButton(wParam);
				return 0;
			}
		case WM_MBUTTONUP:
			{
				if (!jds.OnMouseDown)
					mButton(wParam);
				return 0;
			}
		case WM_RBUTTONUP:
			{
				if (!jds.OnMouseDown)
					rButton(wParam);
				return 0;
			}
		case WM_LBUTTONDOWN:
			{
				if (jds.OnMouseDown)
					lButton(wParam);
				return 0;
			}
		case WM_MBUTTONDOWN:
			{
				if (jds.OnMouseDown)
					mButton(wParam);
				return 0;
			}
		case WM_RBUTTONDOWN:
			{
				if (jds.OnMouseDown)
					rButton(wParam);
				return 0;
			}
	}
	return DefWindowProc(hwnd,message,wParam,lParam);
}

void lButton(WPARAM wParam)
{
	if (wParam & MK_LBUTTON)
		wParam = wParam ^ MK_LBUTTON;
	switch(wParam)
	{
		case MK_CONTROL:
			executeCommand(jds.CtrlMButton1);
			break;
		case MK_SHIFT:
			executeCommand(jds.ShiftMButton1);
			break;
		default:
			executeCommand(jds.MButton1);
			break;
	}
	return;
}

void mButton(WPARAM wParam)
{
	if (wParam & MK_MBUTTON)
		wParam = wParam ^ MK_MBUTTON;
	switch(wParam)
	{
		case MK_CONTROL:
			executeCommand(jds.CtrlMButton3);
			break;
		case MK_SHIFT:
			executeCommand(jds.ShiftMButton3);
			break;
		default:
			executeCommand(jds.MButton3);
			break;
	}
}

void rButton(WPARAM wParam)
{
	if (wParam & MK_RBUTTON)
		wParam = wParam ^ MK_RBUTTON;
	switch(wParam)
	{
		case MK_CONTROL:
			executeCommand(jds.CtrlMButton2);
			break;
		case MK_SHIFT:
			executeCommand(jds.ShiftMButton2);
			break;
		default:
			executeCommand(jds.MButton2);
			break;
	}
	return;
}

void executeCommand(char *command)
{
	char args[MAXLEN];
	char tmp[MAXLEN], tmpcmd[MAXLEN];
	char *token;

	if(strlen(command)<1)
		return;

	strcpy(tmp, command);
	token = strtok(tmp, "*");
		if (token != NULL) strcpy(tmpcmd, token);
	token = strtok(NULL, "*");
		if (token != NULL) strcpy(args, token);

	if (tmpcmd[0] == '!')
		ParseBangCommand(hMainWnd,tmpcmd,args);
	else
	{
		SHELLEXECUTEINFO si;
		char workDirectory[MAX_PATH];
		char drive[_MAX_DRIVE];
		char dir[_MAX_DIR];

		_splitpath(tmpcmd, drive, dir, NULL, NULL);
		strcpy(workDirectory, drive);
		strcat(workDirectory, dir);

		memset(&si, 0, sizeof(si));
		si.cbSize = sizeof(SHELLEXECUTEINFO);
		si.lpDirectory = workDirectory;
		si.lpVerb = NULL;
		si.nShow = 1;
		si.fMask = SEE_MASK_DOENVSUBST;
		si.lpFile = tmpcmd;
		si.lpParameters = args;
		ShellExecuteEx(&si);
	}
}

void setWorkArea(void)
{
	int minSize = 100;//Allowing a minimum of 100x100 desktop area. Anything less is even more rediculous then that, plus impossible to access a max'd window.
	RECT r;
	r.top = 0;
	r.left = 0;
	r.bottom = screenHeight;
	r.right = screenWidth;
	
	if (jds.WorkArea)
	{
		r.left = waLeft<0 ? screenWidth+waLeft:waLeft;
			if (r.left >= screenWidth-minSize) r.left = 0;

		r.right = waRight<0 ? screenWidth+waRight:waRight;
			if (r.right <= r.left+minSize) r.right = screenWidth;

		r.top = waTop<0 ? screenHeight+waTop:waTop;
			if (r.top >= screenHeight-minSize) r.top = 0;

		r.bottom = waBottom<0 ? screenHeight+waBottom:waBottom;
			if (r.bottom <= r.top+minSize) r.bottom = screenHeight;
	}
	SystemParametersInfo(SPI_SETWORKAREA,0,&r,SPIF_SENDCHANGE);
	return;
}

void resetWorkArea(void)
{
	RECT r;
	r.top = 0;
	r.left = 0;
	r.bottom = screenHeight;
	r.right = screenWidth;
	SystemParametersInfo(SPI_SETWORKAREA,0,&r,SPIF_SENDCHANGE);
	return;
}

void jDeskSetWorkArea(HWND caller, char *args)
{
	caller=caller;
	jds.WorkArea = TRUE;

	args = strtok(args, " ,");
		if (args != NULL)
			waLeft = atoi(args);
	args = strtok(NULL, " ,");
		if (args != NULL)
			waRight = atoi(args);
	args = strtok(NULL, " ,");
		if (args != NULL)
			waTop = atoi(args);
	args = strtok(NULL, " ,");
		if (args != NULL)
			waBottom = atoi(args);
	setWorkArea();
}

void jDeskResetWorkArea(void)
{
	waLeft=0;
	waRight=-50;
	waTop=0;
	waBottom=-50;
	jds = ReadSettings();
	setWorkArea();
}
